<?php

header("Content-Type:application/json; charset=UTF-8");

/* (C) Fabrice Sincère

Simulation du déplacement d'un bus sur un circuit fermé
version 0.09

format json : retourne timestamp, lat (°), long (°) , vitesse (km/h) et compteur au format xxx.xx km
ex: {"time":1507291158.3383,"vitesse":28,"lat":44.932417280683,"long":4.9185179610103,"compteur":"716.40"}


PHP >= 5 fonction json_encode()
PHP5 windows : [OK]
PHP5 linux :   [OK]

*/

function distance_GPS($latitude1, $longitude1, $latitude2, $longitude2) {

	// distance en m entre deux coordonnées GPS (en degrés)
	// biblio : http://www.01net.com/astuces/astuce-excel-calculez-la-distance-entre-deux-points-de-la-terre-555908.html
	// https://fr.wikipedia.org/wiki/Orthodromie
	// http://www.lexilogos.com/calcul_distances.htm

	$distance = acos(sin(deg2rad($latitude1))*sin(deg2rad($latitude2)) + cos(deg2rad($latitude1))*cos(deg2rad($latitude2))*cos(deg2rad($longitude1-$longitude2)))*6371000.0;

	// exemple : Paris - New York
	// distance_GPS(48.85, 2.35, 40.7166, -74.00) --> 5836 km environ
	return $distance; // en mètres
}

function distance_circuit_ferme($tableau) {

	// retourne la distance d'un circuit fermé

	/* exemple de circuit polygonal :
	$circuit1 = array();
	$circuit1[0] = array('lat' => 44.93271, 'long' => 4.91857); // distance 0 -> 1 : 368.09
	$circuit1[1] = array('lat' => 44.93230, 'long' => 4.92321); // distance 1 -> 2 : 303.60
	$circuit1[2] = array('lat' => 44.92959, 'long' => 4.92274); // distance 2 -> 3 : 369.03
	$circuit1[3] = array('lat' => 44.93001, 'long' => 4.91809); // distance 3 -> 0 : 302.59
	résultat : distance totale : 1343.32
	 */

	$taille = count($tableau);
	$distance = 0.0;
	for ($i = 0; $i < $taille; $i++) {
		$distance += distance_GPS($tableau[$i]['lat'], $tableau[$i]['long'] ,$tableau[($i+1)%$taille]['lat'], $tableau[($i+1)%$taille]['long']);
	}
	return $distance; // en mètres

}

function distance($tableau, $temps){

	// $tableau
	/* par ex :
	$profil_distance = array();
	$profil_distance[0] = array('time' => 0, 'distance' => 0.0);
	$profil_distance[1] = array('time' => 2.0, 'distance' => 1.85);
	$profil_distance[2] = array('time' => 4.0, 'distance' => 7.41);
	...
	$profil_distance[36] = array('time' => 180, 'distance' => 1343.32);
	*/

	// $temps en seconde (ramené sur la période)

	// on recherche les indices qui bornent le temps donné
	// ex : $temps = 2.5 =>  indice supérieur 2


	for($indice_superieur = 1; $indice_superieur < count($tableau) -1; $indice_superieur++){

		// recherche par balayage (une dichotomie serait plus efficace...)
		if ($tableau[$indice_superieur]['time'] >= $temps) break;
	}

	// interpolation linéaire
	$distance = $tableau[$indice_superieur - 1]['distance'] + ($tableau[$indice_superieur]['distance'] - $tableau[$indice_superieur - 1]['distance'])/($tableau[$indice_superieur]['time']-$tableau[$indice_superieur -1]['time'])* ($temps -
	$tableau[$indice_superieur -1]['time']);

	// ex : $temps = 2.5
	// 2.0 -> 1.85
	// 4.0 -> 7.41
	// interpolation linéaire : 3.24 m

    

	return $distance;  // en m  (position dépuis le début du circuit)
}

function vitesse($tableau, $temps){
	// $tableau
	/* par ex :
	$profil_distance = array();
	$profil_distance[0] = array('time' => 0, 'distance' => 0.0);
	$profil_distance[1] = array('time' => 2.0, 'distance' => 1.85);
	$profil_distance[2] = array('time' => 4.0, 'distance' => 7.41);
	...
	$profil_distance[36] = array('time' => 180, 'distance' => 1343.32);
	*/

	// $temps en seconde (ramené sur la période)

	// on estime la vitesse par distance(t) - distance(t-1)
	// ou  distance(t+1) - distance(t)

	if ($temps > 1.0) {
		$res = 3.6*(distance($tableau, $temps) - distance($tableau, $temps-1)); // en km/h
	}
	else {
		$res = 3.6*(distance($tableau, $temps+1) - distance($tableau, $temps)); // en km/h
	}

	// ajout d'un bruit de mesure de +/- 1 km/h
	$res += rand (0, 2) - 1;
	$res = (int) $res;
	if ($res <=1) $res = 0;

	return $res;  // arrondi à l'entier
}


function tempscycle($tableau){

	// tranforme le timestamp en temps cycle, cad
	// une valeur entre 0 et la période du cycle (180 dans l'ex ci-dessous)
    
	// $tableau
	/* par ex :
	$profil_distance = array();
	$profil_distance[0] = array('time' => 0, 'distance' => 0.0);
	$profil_distance[1] = array('time' => 2.0, 'distance' => 1.85);
	$profil_distance[2] = array('time' => 4.0, 'distance' => 7.41);
	...
	$profil_distance[36] = array('time' => 180, 'distance' => 1343.32);
	*/

	$periode = (int) $tableau[count($tableau) -1]['time']; // ex 180

	$time = microtime(False); // retourne le timestamp avec les microsecondes
	// chaîne de caractères au format "msec sec"
	// ex : "0.08060000 1505302602"

	list($usec, $sec) = explode(" ", $time);
	$usec = (float)$usec; // ex : 0.0806
	$sec = (int)$sec; // ex : 1505302602

	$timestamp = $sec + $usec;

	$timemodulo = $sec % $periode + $usec; // valeur comprise entre 0.0 et $periode
	// ex : 2.0806

    


	return array($timemodulo, $timestamp, $periode);
}





function distance_vers_gps($tableau, $distance){

	/* exemple de circuit polygonal :
	$circuit1 = array();
	point 0
	$circuit1[0] = array('lat' => 44.93271, 'long' => 4.91857); // distance 0 -> 1 : 368.09
	point 1
	$circuit1[1] = array('lat' => 44.93230, 'long' => 4.92321); // distance 1 -> 2 : 303.60
	$circuit1[2] = array('lat' => 44.92959, 'long' => 4.92274); // distance 2 -> 3 : 369.03
	$circuit1[3] = array('lat' => 44.93001, 'long' => 4.91809); // distance 3 -> 0 : 302.59
	distance totale : 1343.32
	*/

	 // on détermine dans quel tronçon on est

	 $distancetotale = distance_circuit_ferme($tableau);
	 if ($distance >= $distancetotale) {
		 $distance -= $distancetotale;
	 }

	 $trajet = 0.0; // en m
	 $index = 0;
	 $taille = count($tableau);

	 while ($trajet <= $distance){
		$trajet += distance_GPS($tableau[$index]['lat'], $tableau[$index]['long'], $tableau[($index+1)%$taille]['lat'], $tableau[($index+1)%$taille]['long']) ;
		$index += 1;
		$index = $index%$taille;
	 }


	//print "Position du prochain point ".$trajet."<br>";

	$prochainpoint = $index%$taille;
	$pointprecedent = ($index +$taille -1)%$taille;

	//print "Prochain point ".$prochainpoint."<br>";


	// interpolation linéaire GPS
	// entre les 2 points qui encadrent la position actuelle

	$longueursegmentencours = distance_GPS($tableau[$prochainpoint]['lat'], $tableau[$prochainpoint]['long'], $tableau[$pointprecedent]['lat'], $tableau[$pointprecedent]['long']) ;

	$distanceactuelleauprochaintpoint = $trajet - $distance;
	//print "distance au prochain point $distanceactuelleauprochaintpoint <br>";

	$latitude = $tableau[$pointprecedent]['lat'] + ($tableau[$prochainpoint]['lat'] - $tableau[$pointprecedent]['lat'])/($longueursegmentencours)*($longueursegmentencours -$distanceactuelleauprochaintpoint);

	//print "latitude $latitude<br>";

	$longitude = $tableau[$pointprecedent]['long'] + ($tableau[$prochainpoint]['long'] - $tableau[$pointprecedent]['long'])/($longueursegmentencours)*($longueursegmentencours -$distanceactuelleauprochaintpoint);

	//print "longitude $longitude<br>";

	// verif pour estimer l'erreur d'interpolation
	// verif OK écart < 1 mm !
	// distance au prochain point

	// $res = distance_GPS($tableau[$prochainpoint]['lat'], $tableau[$prochainpoint]['long'], $latitude, $longitude);
	// print "distance au prochain point $res<br>";

	return array($latitude, $longitude);
}


function compteur($distance, $periode, $timestamp, $temps, $longueurcircuit){
    // compteur de km au format xxx.xx km
    
    $nb_cycle = ($timestamp - $temps)/$periode; // nb entier
    $compteur = $nb_cycle*$longueurcircuit + $distance;
    $compteur *= 0.001; // en km
    $compteur = $compteur - intval($compteur*0.001)*1000;  // modulo 1000
    $compteur = sprintf("%06.2f", $compteur); // formatage
    return $compteur;
}


// définition du circuit fermé polygonal
// liste des coordonnées
/*
44.93271, 4.91857   // Valence rue mozart // feux tricolore
44.93230, 4.92321   // rue Biberach
44.92959, 4.92274   // rue Verdi
44.93001, 4.91809   // Boulevard Churchill // feux tricolore
*/

$circuit1 = array();
$circuit1[0] = array('lat' => 44.93271, 'long' => 4.91857);
$circuit1[1] = array('lat' => 44.93230, 'long' => 4.92321);
$circuit1[2] = array('lat' => 44.92959, 'long' => 4.92274);
$circuit1[3] = array('lat' => 44.93001, 'long' => 4.91809);


// définition du profil distance en fonction du temps
// arrêt au feu tricolore
// vitesse de croisière 50 km/h
// accélération constance 0 - 50 km/h en 15 secondes
// 50 km/h <=> 13.889 m/s
// a = dv/dt = 13.89/15 = 0.9259 m/s2
// v = a*t = dx/dt => x = a*t^2/2
// t = 15 s -> x = 104.17 mètres

// tableau distance en fonction du temps

$profil_distance = array();

// démarrage
// 0 à 50 km/h

$profil_distance[0] = array('time' => 0, 'distance' => 0.0);
$profil_distance[1] = array('time' => 2.0, 'distance' => 1.85);
$profil_distance[2] = array('time' => 4.0, 'distance' => 7.41);
$profil_distance[3] = array('time' => 6.0, 'distance' => 16.67);
$profil_distance[4] = array('time' => 8.0, 'distance' => 29.63);
$profil_distance[5] = array('time' => 10.0, 'distance' => 46.29);
$profil_distance[6] = array('time' => 12.0, 'distance' => 66.66);
$profil_distance[7] = array('time' => 14.0, 'distance' => 90.74);
$profil_distance[8] = array('time' => 15.0, 'distance' => 104.17);

// vitesse permanente
// 1343.32 - 302.59 - 104.17 =  936.56
// 936.56 - 104.17 = 832.39 m à 50 km/h
// 59.93 s
// 15 + 59.93 = 74.93

$profil_distance[9] = array('time' => 74.93, 'distance' => 936.56);

// freinage

$profil_distance[10] = array('time' => 75.93, 'distance' => 1040.73 - 90.74);
$profil_distance[11] = array('time' => 77.93, 'distance' => 1040.73 - 66.66);
$profil_distance[12] = array('time' => 79.93, 'distance' => 1040.73 - 46.29);
$profil_distance[13] = array('time' => 81.93, 'distance' => 1040.73 - 29.63);
$profil_distance[14] = array('time' => 83.93, 'distance' => 1040.73 - 16.67);
$profil_distance[15] = array('time' => 85.93, 'distance' => 1040.73 - 7.41);
$profil_distance[16] = array('time' => 87.93, 'distance' => 1040.73 - 1.85);
// 1343.32 - 302.59 = 1040.73
$profil_distance[17] = array('time' => 89.93, 'distance' => 1040.73);

// arrêt
// attente 20 s
$profil_distance[18] = array('time' => 109.93, 'distance' => 1040.73);


// démarrage
$profil_distance[19] = array('time' => 111.93, 'distance' => 1040.73 + 1.85);
$profil_distance[20] = array('time' => 113.93, 'distance' => 1040.73 + 7.41);
$profil_distance[21] = array('time' => 115.93, 'distance' => 1040.73 + 16.67);
$profil_distance[22] = array('time' => 117.93, 'distance' => 1040.73 + 29.63);
$profil_distance[23] = array('time' => 119.93, 'distance' => 1040.73 + 46.29);
$profil_distance[24] = array('time' => 121.93, 'distance' => 1040.73 + 66.66);
$profil_distance[25] = array('time' => 123.93, 'distance' => 1040.73 + 90.74);
// 1040.73 + 104.17 = 1144.90
$profil_distance[26] = array('time' => 124.93, 'distance' => 1144.90);

// vitesse permanente
// 1343.32 - 104.17 =  1239.15
// 1239.15 - 1144.90 = 94.25 m à 50 km/h
// 6.79 s
// 124.93 + 6.79 = 131.72

$profil_distance[27] = array('time' => 131.72, 'distance' => 1239.15);

// freinage

$profil_distance[28] = array('time' => 132.72, 'distance' => 1343.32 - 90.74);
$profil_distance[29] = array('time' => 134.72, 'distance' => 1343.32 - 66.66);
$profil_distance[30] = array('time' => 136.72, 'distance' => 1343.32 - 46.29);
$profil_distance[31] = array('time' => 138.72, 'distance' => 1343.32 - 29.63);
$profil_distance[32] = array('time' => 140.72, 'distance' => 1343.32 - 16.67);
$profil_distance[33] = array('time' => 142.72, 'distance' => 1343.32 - 7.41);
$profil_distance[34] = array('time' => 144.72, 'distance' => 1343.32 - 1.85);
$profil_distance[35] = array('time' => 146.72, 'distance' => 1343.32);

// arrêt
// attente 33.28 s pour une avoir une période entière (180 secondes exactement)

$profil_distance[36] = array('time' => 180, 'distance' => 1343.32);
//print_r($profil_distance);

// fin de la définition du profil de distance


list($temps, $TIMESTAMP, $periode) = tempscycle($profil_distance);
//print $temps."<br>";
//print $periode."<br>";

$distance = distance($profil_distance, $temps);
//print $distance."<br>";

$longueurcircuit = distance_circuit_ferme($circuit1);
//print $longueurcircuit."<br>";

$COMPTEUR = compteur($distance, $periode, $TIMESTAMP, $temps, $longueurcircuit);

$VITESSE = vitesse($profil_distance, $temps);

list($LATITUDE, $LONGITUDE) = distance_vers_gps($circuit1, $distance);

$arr = array('time' => $TIMESTAMP, 'vitesse' => $VITESSE, 'lat' => $LATITUDE, 'long' => $LONGITUDE, 'compteur' => $COMPTEUR);

$myJSON = json_encode($arr);

print $myJSON;
// ex: {"time":1507291158.3383,"vitesse":28,"lat":44.932417280683,"long":4.9185179610103,"compteur":"716.408"}



?>
